/* Copyright © 2013, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>

#include <glaze.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void glenv_query (void);

typedef GLubyte * (get_string_type_t) (GLenum);

void
glenv_query (void)
{
	printf ("--vendor=\"%s\"\n", glGetString (GL_VENDOR));
	printf ("--renderer=\"%s\"\n", glGetString (GL_RENDERER));
	printf ("--version=\"%s\"\n", glGetString (GL_VERSION));
	printf ("--shading-language-version=\"%s\"\n", glGetString (GL_SHADING_LANGUAGE_VERSION));
	printf ("--extensions=\"%s\"\n", glGetString (GL_EXTENSIONS));

	exit (0);
}

static int
glenv_num_extensions (void)
{
	/* FIXME */
	return 0;
}

static int
glenv_major_version (void)
{
	const char *version = getenv ("GLENV_GL_VERSION");
	if (version == NULL)
		return 0;

	return atoi (version);
}

static int
glenv_minor_version (void)
{
	const char *version = getenv ("GLENV_GL_VERSION");
	const char *decimal;

	if (version == NULL)
		return 0;

	decimal = strchr (version, '.');

	if (decimal == NULL)
		return 0;

	return atoi (decimal + 1);
}

void
glGetDoublev (GLenum pname, GLdouble *params)
{
	switch (pname) {
	case GL_NUM_EXTENSIONS:
		*params = glenv_num_extensions();
		break;
	case GL_MAJOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_major_version();
			return;
		}
		break;
	case GL_MINOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_minor_version();
			return;
		}
		break;
	}

	GLAZE_DEFER (glGetDoublev, pname, params);
}

void
glGetFloatv (GLenum pname, GLfloat *params)
{
	switch (pname) {
	case GL_NUM_EXTENSIONS:
		*params = glenv_num_extensions();
		break;
	case GL_MAJOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_major_version();
			return;
		}
		break;
	case GL_MINOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_minor_version();
			return;
		}
		break;
	}

	GLAZE_DEFER (glGetFloatv, pname, params);
}

void
glGetIntegerv (GLenum pname, GLint *params)
{
	switch (pname) {
	case GL_NUM_EXTENSIONS:
		*params = glenv_num_extensions();
		break;
	case GL_MAJOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_major_version();
			return;
		}
		break;
	case GL_MINOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_minor_version();
			return;
		}
		break;
	}

	GLAZE_DEFER (glGetIntegerv, pname, params);
}

void
glGetInteger64v (GLenum pname, GLint64 * params)
{
	switch (pname) {
	case GL_NUM_EXTENSIONS:
		*params = glenv_num_extensions();
		break;
	case GL_MAJOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_major_version();
			return;
		}
		break;
	case GL_MINOR_VERSION:
		if (getenv ("GLENV_GL_VERSION")) {
			*params = glenv_minor_version();
			return;
		}
		break;
	}

	GLAZE_DEFER (glGetInteger64v, pname, params);
}

const GLubyte *
glGetString (GLenum name)
{
	const GLubyte *ret;

	switch (name) {
	case GL_VENDOR:
		ret = (GLubyte *) getenv ("GLENV_GL_VENDOR");
		if (ret)
			return ret;
		break;
	case GL_RENDERER:
		ret = (GLubyte *) getenv ("GLENV_GL_RENDERER");
		if (ret)
			return ret;
		break;
	case GL_VERSION:
		ret = (GLubyte *) getenv ("GLENV_GL_VERSION");
		if (ret)
			return ret;
		break;
	case GL_SHADING_LANGUAGE_VERSION:
		ret = (GLubyte *) getenv ("GLENV_GL_SHADING_LANGUAGE_VERSION");
		if (ret)
			return ret;
		break;
	case GL_EXTENSIONS:
		ret = (GLubyte *) getenv ("GLENV_GL_EXTENSIONS");
		if (ret)
			return ret;
		break;
	}

	GLAZE_DEFER_WITH_RETURN (ret, glGetString, name);

	return ret;
}

const GLubyte *
glGetStringi (GLenum name, GLuint index)
{
	const GLubyte *ret;

	switch (name) {
	case GL_EXTENSIONS:
		/* FIXME */
		break;
	case GL_SHADING_LANGUAGE_VERSION:
		if (index == 0) {
			ret = (GLubyte *) getenv ("GLENV_GL_SHADING_LANGUAGE_VERSION");
			if (ret)
				return ret;
		}
		break;
	}

	GLAZE_DEFER_WITH_RETURN (ret, glGetStringi, name, index);

	return ret;
}
